class Seep::Doc
  attr_reader :dom, :url

  def initialize(url, html)
    @url = url
    @dom = Nokogiri::HTML.parse(html)
  end

  def links
    @links ||= [].tap do |links|
      @dom.search("a").each do |anchor|
        link = compute_url( anchor.get_attribute("href") )
        links << link unless link.nil? or link == url
      end
    end
  end

  def images
    @images ||= [].tap do |images|
      @dom.search("img").each do |image|
        link = compute_url( image.get_attribute("src") )
        images << link unless link.nil?
      end
    end
  end

  def urls
    links + images
  end

  def compute_url( relative_url )
    return nil if relative_url.nil?
    url = begin
      URI.join( @url, relative_url ).to_s
    rescue
      URI.join( @url, URI.escape(relative_url) ).to_s
    end
    url =~ /^http/i ? url : nil
  end

  def inspect
    "#<Seep::Doc #{url} links: #{links.count}, images: #{images.count}>"
  end
end
