class Seep::Fetcher
  AGENT = 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2'
  
  attr_reader :curb, :size, :body
  attr_accessor :url, :max_file_size
  attr_accessor :request_headers, :response_headers

  def initialize(url, options = {})
    self.url              = url
    self.request_headers  = {}.tap do |head|
      head['User-Agent']  = options[:user_agent] || AGENT
    end
    self.response_headers = {}
    self.max_file_size    = options[:max_file_size] || 1_048_576 # 1MB
    
    @curb                 = Curl::Easy.new(url)
    @curb.follow_location = true
    @curb.max_redirects   = options[:max_redirects] || 5

    register_on_header!
    register_on_body!
  end

  def content_type
    response_headers['Content-Type']
  end

  def dest_url
    curb.last_effective_url || url
  end

  def open(redirect = 0)
    @body = ""; @size = 0
    curb.headers = request_headers
    curb.perform
    self
  end

  def inspect
    "#<Seep::Fetcher #{ content_type.nil? ? dest_url : content_type + ' ' + dest_url }>"
  end

  def export(path)
    File.open(path, 'w') do |file|
      file.write(@body)
    end
  end

  def ext
    case content_type
    when "image/jpeg"; ".jpg"
    when "image/png" ; ".png"
    when "image/gif" ; ".gif"
    when "text/html" ; ".html"
    when "text/plain"; ".txt"
    else; ""
    end
  end

  def image?
    (!! content_type =~ /^image/) and to_image.valid?
  end

  def to_image
    @image ||= Seep::Image.new(body)
  end

  def doc?
    content_type == "text/html"
  end

  def to_doc
    @doc ||= Seep::Doc.new(url, body)
  end

  def self.open(url, options = {})
    self.new(url, options).open
  end

  def register_on_header!
    curb.on_header do |header|
      key, value = header.split(":", 2)
      unless key.nil? or value.nil?
        key.strip!; value.strip!
        @size = value.to_i if key == "Content-Length"
        response_headers[key] = value
      end
      if @size > max_file_size
        @size = -1
      else
        header.length
      end
    end
  end

  def register_on_body!
    curb.on_body do |body|
      @body += body
      @size  = @body.length
      if @size > max_file_size
        @size = -1
      else
        body.length
      end
    end
  end
end
